@@ -3,20 +3,12 @@ import warnings
 
 import cryptography
 from cryptography import x509
-from cryptography.hazmat.primitives.asymmetric import rsa, dsa, ec, dh
-from cryptography.hazmat.primitives.asymmetric import ed25519, x25519
-from cryptography.hazmat.primitives.asymmetric import x448, ed448
 from cryptography.hazmat.primitives import serialization
+from cryptography.hazmat.primitives.asymmetric import dh, dsa, ec, ed448, ed25519, rsa, x448, x25519
 
-from .rsakeys import fermat
-from .rsakeys import pattern
-from .rsakeys import roca
-from .rsakeys import rsainvalid
-from .rsakeys import sharedprimes
-from .rsakeys import smallfactors
-from .rsakeys import xzbackdoor
-from .rsakeys import rsawarnings
 from .allkeys import blocklist
+from .rsakeys import (fermat, pattern, roca, rsainvalid, rsawarnings, sharedprimes, smallfactors,
+                      xzbackdoor)
 
 # List of available checks
 defaultchecks = {
@@ -91,6 +83,10 @@ def _checkkey(key, checks):
         r["results"] = checkrsa(r["n"], e=r["e"], checks=checks)
     elif isinstance(key, ec.EllipticCurvePublicKey):
         r["type"] = "ec"
+        r["curve"] = key.curve.name
+        # shorter names for standard curves
+        if r["curve"].startswith("sec") and r["curve"].endswith("r1"):
+            r["curve"] = r["curve"][3:-2]
         r["x"] = key.public_numbers().x
         r["y"] = key.public_numbers().y
         r["results"] = checkall(r["x"], checks=checks)
@@ -109,6 +105,7 @@ def _checkkey(key, checks):
         ),
     ):
         r["type"] = "ec"
+        r["curve"] = str(type(key).__name__).lower()[:-9]
         # For Ed25519 the raw key is the x coordinate
         x_b = key.public_bytes(
             encoding=serialization.Encoding.Raw,
@@ -163,74 +160,88 @@ def checkall(x, checks=defaultchecks.keys()):
     return results
 
 
+def _reterr(rtype, ex):
+    return {"type": rtype, "results": {}, "exception": ex.__class__.__name__, "errmsg": str(ex)}
+
+
 def checkpubkey(rawkey, checks=defaultchecks.keys()):
     try:
         key = serialization.load_pem_public_key(rawkey.encode())
-    except ValueError:
-        # happens e.g. on partial inputs
-        return {"type": "unparseable", "results": {}}
+    except ValueError as e:
+        # ValueError: defect inputs (partial keys, ASN.1 errors), ECDSA with explicit curves,
+        #             unknown key types (e.g., ML-DSA)
+        return _reterr("unparseable", e)
+    except cryptography.exceptions.UnsupportedAlgorithm as e:
+        # UnsupportedAlgorithm: Unsupported elliptic curves
+        return _reterr("unsupported", e)
     return _checkkey(key, checks)
 
 
 def checkprivkey(rawkey, checks=defaultchecks.keys()):
     try:
         priv = serialization.load_pem_private_key(rawkey.encode(), password=None)
-    except ValueError:
-        # happens on invalid values, e.g. p=q
-        return {"type": "unparseable", "results": {}}
-    except cryptography.exceptions.UnsupportedAlgorithm:
-        # happens e.g. with unsupported curves
-        return {"type": "unparseable", "results": {}}
-    except TypeError:
-        # happens on keys with passwords
-        return {"type": "unparseable", "results": {}}
+    except ValueError as e:
+        # ValueError: defect inputs, unknown key types, explicit curves
+        return _reterr("unparseable", e)
+    except (cryptography.exceptions.UnsupportedAlgorithm, TypeError) as e:
+        # UnsupportedAlgorithm: Unsupported elliptic curves
+        # TypeError: passwords-protected keys
+        return _reterr("unsupported", e)
     return _checkkey(priv.public_key(), checks)
 
 
 def checkcrt(rawcert, checks=defaultchecks.keys()):
     try:
         crt = x509.load_pem_x509_certificate(rawcert.encode())
-    except (ValueError, cryptography.x509.base.InvalidVersion):
-        return {"type": "unparseable", "results": {}}
+    except (ValueError, cryptography.x509.base.InvalidVersion) as e:
+        return _reterr("unparseable", e)
     try:
-        return _checkkey(crt.public_key(), checks)
-    except cryptography.exceptions.UnsupportedAlgorithm:
-        # happens e.g. with PSS keys
-        return {"type": "unsupported", "results": {}}
-    except (ValueError, NotImplementedError):
-        # happens e.g. with ECDSA custom curves
-        return {"type": "unsupported", "results": {}}
+        pubkey = crt.public_key()
+    except (cryptography.exceptions.UnsupportedAlgorithm, ValueError, NotImplementedError) as e:
+        # ValueError: unknown key types, explicit curves
+        # UnsupportedAlgorithm: unsupported curves
+        # NotImplementedError: ? (possibly certificate extension issues)
+        return _reterr("unsupported", e)
+    return _checkkey(pubkey, checks)
 
 
 def checkcsr(rawcsr, checks=defaultchecks.keys()):
     try:
         csr = x509.load_pem_x509_csr(rawcsr.encode())
-    except ValueError:
-        return {"type": "unparseable", "results": {}}
-    return _checkkey(csr.public_key(), checks)
+    except (ValueError, cryptography.x509.base.InvalidVersion) as e:
+        return _reterr("unparseable", e)
+    try:
+        pubkey = csr.public_key()
+    except (cryptography.exceptions.UnsupportedAlgorithm, ValueError) as e:
+        # ValueError: unknown key types, explicit curves
+        # UnsupportedAlgorithm: unsupported curves
+        return _reterr("unsupported", e)
+    return _checkkey(pubkey, checks)
 
 
 def checksshprivkey(sshkey, checks=defaultchecks.keys()):
     try:
         pkey = serialization.load_ssh_private_key(sshkey.encode(), password=None)
-    except ValueError:
-        # happens e.g. on password-protected keys
-        return {"type": "unsupported", "results": {}}
-    except cryptography.exceptions.UnsupportedAlgorithm:
-        # happens e.g. on pre-standard sk-ssh-ed25519@openssh.com keys
-        return {"type": "unsupported", "results": {}}
+    except ValueError as e:
+        # ValueError: defect keys
+        return _reterr("unparseable", e)
+    except (cryptography.exceptions.UnsupportedAlgorithm, TypeError) as e:
+        # UnsupportedAlgorithm: unsupported key types, e.g., sk-*
+        # TypeError: password-protected keys
+        return _reterr("unsupported", e)
     return _checkkey(pkey.public_key(), checks)
 
 
 def checksshpubkey(sshkey, checks=defaultchecks.keys()):
     try:
         pkey = serialization.load_ssh_public_key(sshkey.encode())
-    except ValueError:
-        # happens e.g. on non-standard DSA keys (!=1024 bit)
-        return {"type": "unsupported", "results": {}}
-    except cryptography.exceptions.UnsupportedAlgorithm:
-        # happens e.g. on pre-standard sk-ssh-ed25519@openssh.com keys
-        return {"type": "unsupported", "results": {}}
+    except ValueError as e:
+        # ValueError: defect keys, non-standard DSA keys (!=1024 bit)
+        return _reterr("unparseable", e)
+    except (cryptography.exceptions.UnsupportedAlgorithm, NotImplementedError) as e:
+        # UnsupportedAlgorithm: unsupported key types
+        # NotImplementedError: unsupported EC point format
+        return _reterr("unsupported", e)
     return _checkkey(pkey, checks)
 
 
